part of '../boxw.dart';

class BoxWDialogController {
  BoxWDialogController._();

  /// Control the dialog count
  static final StreamController<int> _currentDialogCountController =
      StreamController.broadcast()
        ..stream.listen((int value) {
          currentDialogCount = currentDialogCount + value;
        });

  /// Listen to the dialog count changed
  static Stream<int> get onDialogCountChanged =>
      _currentDialogCountController.stream;

  /// Get current dialog number
  static int currentDialogCount = 0;
}

Future<T?> boxWDialog<T>({
  /// BuildContext
  required BuildContext context,

  /// String | Widget
  ///
  /// Title
  dynamic title,

  /// String | Widget
  ///
  /// Content of the dialog
  dynamic content,
  double? width,
  double? height,

  /// Show the close button, default is depending on the [title] widget and [barrierDismissible].
  bool? showCloseButton,

  /// Buttons with configs. When you want to close this dialog, you should close
  /// with this [context].
  List<Buttons> Function(BuildContext context)? buttons,
  EdgeInsetsGeometry buttonsPadding = const EdgeInsets.only(top: 20),
  bool isContentAboveButtons = true,
  bool barrierDismissible = true,
  double? dialogRadius,
  final Color? backgroundColor,
  final Color? borderColor,
  final double? elevation,
  final Color? shadowColor,
  final Color? surfaceTintColor,
}) {
  // Do not show the close button when barrierDismissible is false
  if (showCloseButton == null && !barrierDismissible) {
    showCloseButton = false;
  }
  return showDialog<T?>(
    barrierDismissible: barrierDismissible,
    context: context,
    builder: (ctx) {
      return BoxWDialog(
        title: title,
        content: content,
        width: width,
        height: height,
        showCloseButton: showCloseButton,
        buttons: buttons == null ? [] : buttons(ctx),
        buttonsPadding: buttonsPadding,
        isContentAboveButtons: isContentAboveButtons,
        dialogRadius: dialogRadius,
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        elevation: elevation,
        shadowColor: shadowColor,
        surfaceTintColor: surfaceTintColor,
      );
    },
  );
}

class BoxWDialog extends StatefulWidget {
  const BoxWDialog({
    super.key,
    this.title,
    this.content,
    this.showCloseButton = false,
    this.closeButtonIcon,
    this.closeButtonBackgroundColor,
    this.buttons = const [],
    this.buttonsPadding = const EdgeInsets.only(top: 20),
    this.isContentAboveButtons = true,
    this.dialogRadius,
    this.width,
    this.height,
    this.constrains = const BoxConstraints(maxWidth: 400, minWidth: 280),
    this.backgroundColor,
    this.borderColor,
    this.elevation,
    this.shadowColor,
    this.surfaceTintColor,
    this.closeButtonColor,
    this.closeButtonBorderColor,
    this.closeButtonBorderWidth = 1,
  });

  /// String | Widget
  final dynamic title;

  /// String | Widget
  final dynamic content;

  final double? width;

  final double? height;

  final BoxConstraints constrains;

  /// Show the close button. Default is false.
  ///
  /// `null` : depends on the [title] widget.
  /// `true` : Force to show the close button.
  /// `false` : Force not to show the close button.
  final bool? showCloseButton;

  /// Use your modified close button.
  final Widget Function(bool isHover)? closeButtonIcon;

  /// Color of the close button circle background.
  final Color? closeButtonBackgroundColor;

  /// Color of the close button icon.
  final Color? closeButtonColor;

  final double closeButtonBorderWidth;
  final Color? closeButtonBorderColor;

  final List<Buttons> buttons;

  /// Padding of the buttons, default is only top 20.
  final EdgeInsetsGeometry buttonsPadding;
  final bool isContentAboveButtons;

  /// Default is `BoxWConfig.dialogRadius = 12`
  final double? dialogRadius;

  final Color? backgroundColor;
  final Color? borderColor;
  final double? elevation;
  final Color? shadowColor;
  final Color? surfaceTintColor;

  @override
  State<BoxWDialog> createState() => _BoxWDialogState();
}

class _BoxWDialogState extends State<BoxWDialog> {
  final buildButtons = <Widget>[];
  bool isHover = false;
  Widget? title;
  Widget? content;

  @override
  void initState() {
    super.initState();

    // Add dialog
    BoxWDialogController._currentDialogCountController.sink.add(1);

    for (final button in widget.buttons) {
      buildButtons.add(
        Flex(
          direction: button.axis,
          mainAxisSize: button.size,
          mainAxisAlignment: button.align,
          children: [
            for (final myButton in button.buttons) Flexible(child: myButton),
          ],
        ),
      );
    }

    if (widget.title != null) {
      title = widget.title is String
          ? Text(
              widget.title!,
              style: const TextStyle(
                fontSize: 20,
                fontWeight: FontWeight.w500,
              ),
            )
          : widget.title!;
    }

    if (widget.content != null) {
      content =
          (widget.content is String) ? Text(widget.content!) : widget.content!;
    }

    if (!mounted) return;

    setState(() {});
  }

  @override
  void dispose() {
    // Remove dialog
    BoxWDialogController._currentDialogCountController.sink.add(-1);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Dialog(
      backgroundColor: widget.backgroundColor,
      elevation: widget.elevation,
      shadowColor: widget.shadowColor,
      surfaceTintColor: widget.surfaceTintColor,
      shape: RoundedRectangleBorder(
        side: BorderSide(
          color: widget.borderColor ??
              BoxWConfig.borderColor ??
              Colors.transparent,
        ),
        borderRadius: BorderRadius.all(
          Radius.circular(widget.dialogRadius ?? BoxWConfig.dialogRadius),
        ),
      ),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          IntrinsicWidth(
            child: Container(
              width: widget.width,
              height: widget.height,
              constraints: const BoxConstraints(maxWidth: 400, minWidth: 280),
              alignment: Alignment.center,
              child: Stack(
                alignment: Alignment.center,
                children: [
                  Padding(
                    padding: const EdgeInsets.all(12.0),
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        if (title != null) title!,

                        // Must set the size for Divider(), if not it will expand to the full width
                        if (title != null)
                          const SizedBox(
                            width: 230,
                            child: Divider(),
                          ),

                        if (content != null && widget.isContentAboveButtons)
                          content!,
                        if (buildButtons.isNotEmpty)
                          Padding(
                            padding: widget.buttonsPadding,
                            child: Column(
                              mainAxisSize: MainAxisSize.min,
                              children: [...buildButtons],
                            ),
                          ),

                        if (content != null && !widget.isContentAboveButtons)
                          content!,
                      ],
                    ),
                  ),
                  if (widget.showCloseButton == true ||
                      (widget.showCloseButton == null && widget.title != null))
                    Positioned.fill(
                      child: Align(
                        alignment: Alignment.topRight,
                        child: Padding(
                          padding: const EdgeInsets.all(4),
                          child: InkWell(
                            customBorder: const CircleBorder(),
                            focusColor: Colors.red,
                            hoverColor: Colors.red,
                            child: BoxWCircle(
                              width: 24,
                              height: 24,
                              padding: EdgeInsets.zero,
                              margin: EdgeInsets.zero,
                              alignment: Alignment.center,
                              borderWidth: widget.closeButtonBorderWidth,
                              borderColor: widget.closeButtonBorderColor,
                              backgroundColor:
                                  widget.closeButtonBackgroundColor ??
                                      Colors.transparent,
                              child: widget.closeButtonIcon != null
                                  ? widget.closeButtonIcon!(isHover)
                                  : Icon(
                                      Icons.close,
                                      color: isHover
                                          ? Colors.white
                                          : widget.closeButtonColor,
                                      size: 16,
                                    ),
                            ),
                            onTap: () {
                              Navigator.pop(context);
                            },
                            onHover: (isHover) {
                              setState(() => this.isHover = isHover);
                            },
                          ),
                        ),
                      ),
                    ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}
