part of '../boxw.dart';

/// ``` dart
/// BoxWNeumorphismIconButton(
///   child: (controller, isClicked) {
///     return AnimatedIcon(
///       icon: AnimatedIcons.add_event,
///       progress: controller,
///       color: const Color.fromARGB(255, 53, 53, 53),
///     );
///   },
/// ),
/// ```
class BoxWNeumorphismIconButton extends StatefulWidget {
  const BoxWNeumorphismIconButton({
    super.key,
    required this.child,
    required this.onPressed,
    this.blackColor = BoxWColors.black,
    this.whiteColor = BoxWColors.white,
    this.borderRadius = 8,
    this.shadowRadius = 8,
    this.shadowSize = 4,
    this.width = 40,
    this.height = 40,
    this.constraints = const BoxConstraints(minHeight: 40, minWidth: 40),
    this.padding,
    this.margin,
  });

  final Color blackColor;
  final Color whiteColor;
  final Widget Function(AnimationController controller) child;
  final Function(bool isPressed) onPressed;
  final double borderRadius;
  final double shadowRadius;
  final double shadowSize;
  final double? width;
  final double? height;
  final BoxConstraints constraints;
  final EdgeInsetsGeometry? padding;
  final EdgeInsetsGeometry? margin;

  @override
  State<BoxWNeumorphismIconButton> createState() =>
      _BoxWNeumorphismIconButtonState();
}

class _BoxWNeumorphismIconButtonState extends State<BoxWNeumorphismIconButton>
    with TickerProviderStateMixin {
  double _turns = 0.0;
  bool _isClicked = false;
  late AnimationController _controller;

  @override
  void initState() {
    _controller = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 400),
    );
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedRotation(
      turns: _turns,
      duration: const Duration(milliseconds: 500),
      child: GestureDetector(
        onTap: () {
          _isClicked = !_isClicked;
          widget.onPressed(_isClicked);
          if (_isClicked) {
            setState(() => _turns += 1 / 4);
            _controller.forward();
          } else {
            setState(() => _turns -= 1 / 4);
            _controller.reverse();
          }
        },
        child: AnimatedContainer(
          duration: const Duration(milliseconds: 500),
          curve: Curves.easeOutExpo,
          width: widget.width,
          height: widget.height,
          constraints: widget.constraints,
          padding: widget.padding,
          margin: widget.margin ?? EdgeInsets.all(widget.shadowSize),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(widget.borderRadius),
            color: widget.whiteColor,
            boxShadow: [
              BoxShadow(
                blurRadius: widget.shadowRadius,
                offset: _isClicked
                    ? Offset(widget.shadowSize, -widget.shadowSize)
                    : Offset(widget.shadowSize, widget.shadowSize),
                color: Colors.grey,
              ),
              BoxShadow(
                blurRadius: widget.shadowRadius,
                offset: _isClicked
                    ? Offset(-widget.shadowSize, widget.shadowSize)
                    : Offset(-widget.shadowSize, -widget.shadowSize),
                color: Colors.white,
              )
            ],
          ),
          child: Center(
            child: widget.child(_controller),
          ),
        ),
      ),
    );
  }
}

class BoxWNeumorphismIcon extends BoxWNeumorphism {
  const BoxWNeumorphismIcon({
    super.key,
    required super.child,
    super.width = 40,
    super.height = 40,
    super.borderRadius = 8,
    super.shadowRadius = 8,
    super.shadowSize = 4,
  });
}

class BoxWNeumorphismText extends StatelessWidget {
  const BoxWNeumorphismText({
    super.key,
    required this.child,
    this.alignment = Alignment.centerLeft,
  });

  final Widget child;
  final Alignment alignment;

  @override
  Widget build(BuildContext context) {
    return BoxWNeumorphism(
      shadowRadius: 1,
      shadowSize: 1,
      alignment: alignment,
      height: 40,
      child: child,
    );
  }
}

class BoxWNeumorphismInput extends StatelessWidget {
  const BoxWNeumorphismInput({
    super.key,
    required this.title,
    required this.onChanged,
  });

  final String title;
  final Function(String value) onChanged;

  @override
  Widget build(BuildContext context) {
    return BoxWNeumorphism(
      shadowRadius: 1,
      shadowSize: 1,
      alignment: Alignment.centerLeft,
      height: 40,
      margin: const EdgeInsets.only(
        top: 12,
        bottom: 8,
        left: 12,
        right: 12,
      ),
      padding: EdgeInsets.zero,
      child: TextFormField(
        style: const TextStyle(fontSize: 14),
        decoration: InputDecoration(
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(8),
            borderSide: const BorderSide(color: BoxWColors.white),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(8),
            borderSide: const BorderSide(color: BoxWColors.white),
          ),
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(8),
            borderSide: const BorderSide(color: BoxWColors.white),
          ),
          disabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(8),
            borderSide: const BorderSide(color: BoxWColors.white),
          ),
          label: Text(
            title,
            style: const TextStyle(
              backgroundColor: BoxWColors.white,
              fontSize: 14,
            ),
          ),
        ),
        onChanged: onChanged,
      ),
    );
  }
}

class BoxWNeumorphism extends StatelessWidget {
  const BoxWNeumorphism({
    super.key,
    required this.child,
    this.blackColor = BoxWColors.black,
    this.whiteColor = BoxWColors.white,
    this.borderRadius = 8,
    this.shadowRadius = 6,
    this.shadowSize = 4,
    this.width,
    this.height,
    this.constraints = const BoxConstraints(minHeight: 40, minWidth: 40),
    this.padding = const EdgeInsets.symmetric(horizontal: 12),
    this.margin,
    this.alignment = Alignment.center,
    this.floatingAlignment,
    this.isFloating = true,
  });

  final Color blackColor;
  final Color whiteColor;
  final Widget child;
  final double borderRadius;
  final double shadowRadius;
  final double shadowSize;
  final double? width;
  final double? height;
  final BoxConstraints constraints;
  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry? margin;
  final Alignment alignment;
  final Alignment? floatingAlignment;
  final bool isFloating;

  BoxWNeumorphism copyWith({
    Color? blackColor,
    Color? whiteColor,
    Widget? child,
    double? borderRadius,
    double? shadowRadius,
    double? shadowSize,
    double? width,
    double? height,
    BoxConstraints? constraints,
    EdgeInsetsGeometry? padding,
    EdgeInsetsGeometry? margin,
    Alignment? alignment,
    Alignment? floatingAlignment,
    bool? isFloating,
  }) =>
      BoxWNeumorphism(
        blackColor: blackColor ?? this.blackColor,
        whiteColor: whiteColor ?? this.whiteColor,
        borderRadius: borderRadius ?? this.borderRadius,
        shadowRadius: shadowRadius ?? this.shadowRadius,
        shadowSize: shadowSize ?? this.shadowSize,
        width: width ?? this.width,
        height: height ?? this.height,
        constraints: constraints ?? this.constraints,
        padding: padding ?? this.padding,
        margin: margin ?? this.margin,
        alignment: alignment ?? this.alignment,
        floatingAlignment: floatingAlignment ?? this.floatingAlignment,
        isFloating: isFloating ?? this.isFloating,
        child: child ?? this.child,
      );

  @override
  Widget build(BuildContext context) {
    final floatingAligmentTemp = floatingAlignment ?? alignment;
    return Container(
      width: width,
      height: height,
      constraints: constraints,
      padding: padding,
      margin:
          margin ?? EdgeInsets.symmetric(horizontal: 12, vertical: shadowSize),
      alignment: isFloating ? alignment : floatingAligmentTemp,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(borderRadius),
        border: Border.all(color: isFloating ? BoxWColors.white : Colors.grey),
        color: BoxWColors.white,
        boxShadow: [
          BoxShadow(
            blurRadius: shadowRadius,
            offset: Offset(shadowSize, shadowSize),
            color: isFloating ? Colors.grey : BoxWColors.white,
          ),
          BoxShadow(
            blurRadius: shadowRadius,
            offset: Offset(-shadowSize, -shadowSize),
            color: isFloating ? Colors.white : BoxWColors.white,
          )
        ],
      ),
      child: child,
    );
  }
}

class BoxWNeumorphismButton extends StatefulWidget {
  const BoxWNeumorphismButton({
    super.key,
    required this.child,
    required this.onPressed,
    this.isEnabled = true,
  });

  final BoxWNeumorphism child;
  final Function() onPressed;
  final bool isEnabled;

  @override
  State<BoxWNeumorphismButton> createState() => _BoxWNeumorphismButtonState();
}

class _BoxWNeumorphismButtonState extends State<BoxWNeumorphismButton> {
  bool _isPressed = false;
  late final Widget child;

  @override
  void initState() {
    if (!widget.isEnabled) _isPressed = true;
    child = widget.child;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: !widget.isEnabled ? null : () => widget.onPressed(),
      onTapDown: !widget.isEnabled
          ? null
          : (details) => setState(() => _isPressed = true),
      onTapUp: !widget.isEnabled
          ? null
          : (details) => setState(() => _isPressed = false),
      onTapCancel:
          !widget.isEnabled ? null : () => setState(() => _isPressed = false),
      child: widget.child.copyWith(isFloating: !_isPressed),
    );
  }
}
